@@ -1,7 +1,4 @@
-from __future__ import division
-from __future__ import absolute_import
-import six
-from six.moves import range
+from __future__ import division, absolute_import
 
 __copyright__ = "Copyright (C) 2009-2013 Andreas Kloeckner"
 
@@ -25,6 +22,9 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 
+import six
+from six.moves import range
+
 from pytools import memoize, memoize_method
 import numpy as np
 
@@ -201,7 +201,8 @@ class Space(object):
         if basis is None:
             basis = int(metric_matrix.shape[0])
 
-        if isinstance(basis, int):
+        from numbers import Integral
+        if isinstance(basis, Integral):
             basis = ["e%d" % i for i in range(basis)]
 
         if metric_matrix is None:
@@ -210,8 +211,7 @@ class Space(object):
         from pytools import all
         if not (
                 len(metric_matrix.shape) == 2
-                and
-                all(dim == len(basis) for dim in metric_matrix.shape)):
+                and all(dim == len(basis) for dim in metric_matrix.shape)):
             raise ValueError("metric_matrix has the wrong shape")
 
         self.basis_names = basis
@@ -419,17 +419,21 @@ class MultiVector(object):
     :class:`sympy.galgebra.GA.MV` object in :mod:`sympy`, especially with
     respect to the supported operators:
 
-    .. csv-table::
-        :header: Operation, Result
-        :widths: 10, 40
-
-        ``A+B``,             Sum of multivectors
-        ``A-B``,             Difference of multivectors
-        ``A*B``,             Geometric product :math:`AB`
-        ``A^B``,             Outer product :math:`A\wedge B` of multivectors
-        ``A|B``,             Inner product :math:`A\cdot B` of multivectors
-        ``A<<B``,            Left contraction :math:`A\lrcorner B` of multivectors
-        ``A>>B``,            Right contraction :math:`A\llcorner B` of multivectors
+    =================== ========================================================
+    Operation           Result
+    =================== ========================================================
+    ``A+B``             Sum of multivectors
+    ``A-B``             Difference of multivectors
+    ``A*B``             Geometric product :math:`AB`
+    ``A^B``             Outer product :math:`A\wedge B` of multivectors
+    ``A|B``             Inner product :math:`A\cdot B` of multivectors
+    ``A<<B``            Left contraction :math:`A\lrcorner B` (``_|``)
+                        of multivectors, also read as ':math:`A` removed from
+                        :math:`B`'.
+    ``A>>B``            Right contraction :math:`A\llcorner B`  (``|_``)of
+                        multivectors, also read as ':math:`A` without
+                        :math:`B`'.
+    =================== ========================================================
 
     .. warning ::
 
@@ -598,12 +602,15 @@ class MultiVector(object):
 
             blade_str = self.space.blade_bits_to_str(bits)
             if bits:
-                terms.append("%s*%s" % (coeff_str, blade_str))
+                terms.append("%s * %s" % (blade_str, coeff_str))
             else:
                 terms.append(coeff_str)
 
         if terms:
-            result = " + ".join(terms)
+            if any(len(t) > 15 for t in terms):
+                result = "\n    " + "\n    + ".join(terms)
+            else:
+                result = " + ".join(terms)
         else:
             result = "0"
 
@@ -918,7 +925,7 @@ class MultiVector(object):
         """
 
         if tol is None:
-            tol = 1e-13
+            tol = 1e-12
 
         new_data = {}
         for bits, coeff in six.iteritems(self.data):
