@@ -1,4 +1,4 @@
-from __future__ import division
+from __future__ import division, absolute_import
 
 __copyright__ = "Copyright (C) 2009-2013 Andreas Kloeckner"
 
@@ -22,7 +22,156 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 
-import traits
+import pymbolic.traits as traits
+
+import six
+from six.moves import range, zip, intern
+
+__doc__ = """
+Expression base class
+---------------------
+
+.. autoclass:: Expression
+
+Sums, products and such
+-----------------------
+
+.. autoclass:: Variable
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: Call
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: CallWithKwargs
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: Subscript
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: Lookup
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: Sum
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: Product
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: Quotient
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: FloorDiv
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: Remainder
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: Power
+    :undoc-members:
+    :members: mapper_method
+
+Shift operators
+---------------
+
+.. autoclass:: LeftShift
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: RightShift
+    :undoc-members:
+    :members: mapper_method
+
+Bitwise operators
+-----------------
+
+.. autoclass:: BitwiseNot
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: BitwiseOr
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: BitwiseXor
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: BitwiseAnd
+    :undoc-members:
+    :members: mapper_method
+
+Comparisons and logic
+---------------------
+
+.. autoclass:: Comparison
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: LogicalNot
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: LogicalAnd
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: LogicalOr
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: If
+    :undoc-members:
+    :members: mapper_method
+
+Code generation helpers
+-----------------------
+
+.. autoclass:: CommonSubexpression
+    :undoc-members:
+    :members: mapper_method
+
+.. autoclass:: cse_scope
+.. autofunction:: make_common_subexpression
+
+Helper functions
+----------------
+
+.. autofunction:: is_zero
+.. autofunction:: is_constant
+.. autofunction:: register_constant_class
+.. autofunction:: unregister_constant_class
+.. autofunction:: variables
+
+Interaction with :mod:`numpy` arrays
+------------------------------------
+
+:mod:`numpy.ndarray` instances are supported anywhere in an expression.
+In particular, :mod:`numpy` object arrays are useful for capturing
+vectors and matrices of :mod:`pymbolic` objects.
+
+.. autofunction:: make_sym_vector
+.. autofunction:: make_sym_array
+"""
+
+
+def disable_subscript_by_getitem():
+    # The issue that was addressed by this could be fixed
+    # in a much less ham-fisted manner, and thus this has been
+    # made a no-op.
+    #
+    # See
+    # https://github.com/inducer/pymbolic/issues/4
+    pass
 
 
 class Expression(object):
@@ -30,8 +179,51 @@ class Expression(object):
     to implicitly construct :class:`Sum`, :class:`Product` and other expressions.
 
     Expression objects are immutable.
+
+    .. attribute:: a
+
+    .. attribute:: attr
+
+    .. attribute:: mapper_method
+
+        The :class:`pymbolic.mapper.Mapper` method called for objects of
+        this type.
+
+    .. method:: __getitem__
+
+    .. automethod:: make_stringifier
+
+    .. automethod:: __eq__
+    .. automethod:: __hash__
+    .. automethod:: __str__
+    .. automethod:: __repr__
+
+    .. automethod:: __repr__
+
+    .. rubric:: Logical operator constructors
+
+    .. automethod:: not_
+    .. automethod:: and_
+    .. automethod:: or_
+
+    .. rubric:: Comparison constructors
+
+    .. automethod:: eq
+    .. automethod:: ne
+    .. automethod:: lt
+    .. automethod:: le
+    .. automethod:: gt
+    .. automethod:: ge
     """
 
+    # {{{ init arg names (override by subclass)
+
+    @property
+    def init_arg_names(self):
+        raise NotImplementedError
+
+    # }}}
+
     # {{{ arithmetic
 
     def __add__(self, other):
@@ -137,7 +329,7 @@ class Expression(object):
             return NotImplemented
 
         if is_zero(other-1):
-            return self
+            return 0
         return Remainder(self, other)
 
     def __rmod__(self, other):
@@ -208,36 +400,81 @@ class Expression(object):
 
     # }}}
 
-    # {{{
+    # {{{ misc
 
     def __neg__(self):
         return -1*self
 
-    def __call__(self, *pars):
-        return Call(self, pars)
+    def __call__(self, *args, **kwargs):
+        if kwargs:
+            return CallWithKwargs(self, args, kwargs)
+        else:
+            return Call(self, args)
+
+    def index(self, subscript):
+        """Return an expression representing ``self[subscript]``.
+
+        .. versionadded:: 2014.3
+        """
 
-    def __getitem__(self, subscript):
         if subscript == ():
             return self
         else:
             return Subscript(self, subscript)
 
+    __getitem__ = index
+
     def attr(self, name):
         """Return a :class:`Lookup` for *name* in *self*.
         """
         return Lookup(self, name)
 
+    @property
+    def a(self):
+        """Provide a spelling ``expr.a.name`` for encoding attribute lookup.
+        """
+        class AttributeLookupCreator(object):
+            def __init__(self, aggregate):
+                self.aggregate = aggregate
+
+            def __getattr__(self, name):
+                return Lookup(self.aggregate, name)
+
+        return AttributeLookupCreator(self)
+
     def __float__(self):
         from pymbolic.mapper.evaluator import evaluate_to_float
         return evaluate_to_float(self)
 
-    def stringifier(self):
-        """Return a :class:`pymbolic.mapper.Mapper` class used to yield
-        a human-readable representation of *self*. Usually a subclass
-        of :class:`pymbolic.mapper.stringifier.StringifyMapper`.
+    def make_stringifier(self, originating_stringifier=None):
+        """Return a :class:`pymbolic.mapper.Mapper` instance that can
+        be used to generate a human-readable representation of *self*. Usually
+        a subclass of :class:`pymbolic.mapper.stringifier.StringifyMapper`.
+
+        :arg originating_stringifier: If provided, the newly created
+            stringifier should carry forward attributes and settings of
+            *originating_stringifier*.
         """
-        from pymbolic.mapper.stringifier import StringifyMapper
-        return StringifyMapper
+        if originating_stringifier is None:
+            stringify_mapper_args = ()
+        else:
+            stringify_mapper_args = (originating_stringifier.constant_mapper,)
+
+        try:
+            stringifier_class_getter = self.stringifier
+        except AttributeError:
+            from pymbolic.mapper.stringifier import StringifyMapper
+            return StringifyMapper(*stringify_mapper_args)
+        else:
+            from warnings import warn
+            warn("%s overrides 'stringifier', which is deprecated. "
+                    "Override 'make_stringifier' instead. "
+                    "Backward compatibility will go away "
+                    "sometime after June 2020."
+                    % type(self).__name__,
+                    DeprecationWarning)
+
+            return stringifier_class_getter()(*stringify_mapper_args)
 
     def __str__(self):
         """Use the :meth:`stringifier` to return a human-readable
@@ -245,15 +482,37 @@ class Expression(object):
         """
 
         from pymbolic.mapper.stringifier import PREC_NONE
-        return self.stringifier()()(self, PREC_NONE)
+        return self.make_stringifier()(self, PREC_NONE)
+
+    def _safe_repr(self, limit=10):
+        if limit <= 0:
+            return "..."
+
+        def strify_child(child, limit):
+            if isinstance(child, tuple):
+                # Make sure limit propagates at least through tuples
+
+                return "(%s%s)" % (
+                        ", ".join(strify_child(i, limit-1) for i in child),
+                        "," if len(child) == 1 else "")
+
+            elif isinstance(child, Expression):
+                return child._safe_repr(limit=limit-1)
+            else:
+                return repr(child)
+
+        initargs_str = ", ".join(
+                strify_child(i, limit-1)
+                for i in self.__getinitargs__())
+
+        return "%s(%s)" % (self.__class__.__name__, initargs_str)
 
     def __repr__(self):
         """Provides a default :func:`repr` based on
         the Python pickling interface :meth:`__getinitargs__`.
         """
-        initargs_str = ", ".join(repr(i) for i in self.__getinitargs__())
+        return self._safe_repr()
 
-        return "%s(%s)" % (self.__class__.__name__, initargs_str)
     # }}}
 
     # {{{ hash/equality interface
@@ -287,6 +546,18 @@ class Expression(object):
             self.hash_value = self.get_hash()
             return self.hash_value
 
+    def __getinitargs__(self):
+        raise NotImplementedError
+
+    def __getstate__(self):
+        return self.__getinitargs__()
+
+    def __setstate__(self, state):
+        # Can't use trivial pickling: hash_value cache must stay unset
+        assert len(self.init_arg_names) == len(state), type(self)
+        for name, value in zip(self.init_arg_names, state):
+            setattr(self, name, value)
+
     # }}}
 
     # {{{ hash/equality backend
@@ -300,9 +571,80 @@ class Expression(object):
 
     # }}}
 
+    # {{{ logical op constructors
+
+    def not_(self):
+        """Return *self* wrapped in a :class:`LogicalNot`.
+
+        .. versionadded:: 2015.2
+        """
+        return LogicalNot(self)
+
+    def and_(self, other):
+        """Return :class:`LogicalAnd` between *self* and *other*.
+
+        .. versionadded:: 2015.2
+        """
+        return LogicalAnd((self, other))
+
+    def or_(self, other):
+        """Return :class:`LogicalOr` between *self* and *other*.
+
+        .. versionadded:: 2015.2
+        """
+        return LogicalOr((self, other))
+
+    # }}}
+
+    # {{{ comparison constructors
+
+    def eq(self, other):
+        """Return a :class:`Comparison` comparing *self* to *other*.
+
+        .. versionadded:: 2015.2
+        """
+        return Comparison(self, "==", other)
+
+    def ne(self, other):
+        """Return a :class:`Comparison` comparing *self* to *other*.
+
+        .. versionadded:: 2015.2
+        """
+        return Comparison(self, "!=", other)
+
+    def le(self, other):
+        """Return a :class:`Comparison` comparing *self* to *other*.
+
+        .. versionadded:: 2015.2
+        """
+        return Comparison(self, "<=", other)
+
+    def lt(self, other):
+        """Return a :class:`Comparison` comparing *self* to *other*.
+
+        .. versionadded:: 2015.2
+        """
+        return Comparison(self, "<", other)
+
+    def ge(self, other):
+        """Return a :class:`Comparison` comparing *self* to *other*.
+
+        .. versionadded:: 2015.2
+        """
+        return Comparison(self, ">=", other)
+
+    def gt(self, other):
+        """Return a :class:`Comparison` comparing *self* to *other*.
+
+        .. versionadded:: 2015.2
+        """
+        return Comparison(self, ">", other)
+
+    # }}}
+
     # {{{ prevent less / greater comparisons
 
-    # /!\ Don't be tempted to resolve these to ComparisonOperator.
+    # /!\ Don't be tempted to resolve these to Comparison.
 
     def __le__(self, other):
         raise TypeError("expressions don't have an order")
@@ -318,6 +660,10 @@ class Expression(object):
 
     # }}}
 
+    def __iter__(self):
+        # prevent infinite loops (e.g. when inserting into numpy arrays)
+        raise TypeError("expression types are not iterable")
+
 
 class AlgebraicLeaf(Expression):
     """An expression that serves as a leaf for arithmetic evaluation.
@@ -336,8 +682,11 @@ class Variable(Leaf):
     """
     .. attribute:: name
     """
+    init_arg_names = ("name",)
+
     def __init__(self, name):
-        self.name = name
+        assert name
+        self.name = intern(name)
 
     def __getinitargs__(self):
         return self.name,
@@ -348,6 +697,11 @@ class Variable(Leaf):
         else:
             return NotImplemented
 
+    def __setstate__(self, val):
+        super(Variable, self).__setstate__(val)
+
+        self.name = intern(self.name)
+
     mapper_method = intern("map_variable")
 
 
@@ -377,15 +731,19 @@ class Call(AlgebraicLeaf):
     """A function invocation.
 
     .. attribute:: function
+
+        A :class:`Expression` that evaluates to a function.
+
     .. attribute:: parameters
+
+        A :class:`tuple` of positional parameters, each element
+        of which is a :class:`Expression` or a constant.
+
     """
 
-    def __init__(self, function, parameters):
-        """
-        :arg function: A :class:`Expression` that evaluates to a function.
-        :arg parameters: A :class:`tuple` of positional paramters.
-        """
+    init_arg_names = ("function", "parameters",)
 
+    def __init__(self, function, parameters):
         self.function = function
         self.parameters = parameters
 
@@ -405,12 +763,83 @@ class Call(AlgebraicLeaf):
     mapper_method = intern("map_call")
 
 
+class CallWithKwargs(AlgebraicLeaf):
+    """A function invocation with keyword arguments.
+
+    .. attribute:: function
+
+        A :class:`Expression` that evaluates to a function.
+
+    .. attribute:: parameters
+
+        A :class:`tuple` of positional parameters, each element
+        of which is a :class:`Expression` or a constant.
+
+    .. attribute:: kw_parameters
+
+        A dictionary mapping names to arguments, , each
+        of which is a :class:`Expression` or a constant,
+        or an equivalent value accepted by the :class:`dict`
+        constructor.
+    """
+
+    init_arg_names = ("function", "parameters", "kw_parameters")
+
+    def __init__(self, function, parameters, kw_parameters):
+        self.function = function
+        self.parameters = parameters
+
+        if isinstance(kw_parameters, dict):
+            self.kw_parameters = kw_parameters
+        else:
+            self.kw_parameters = dict(kw_parameters)
+
+        try:
+            arg_count = self.function.arg_count
+        except AttributeError:
+            pass
+        else:
+            if len(self.parameters) != arg_count:
+                raise TypeError("%s called with wrong number of arguments "
+                        "(need %d, got %d)" % (
+                            self.function, arg_count, len(parameters)))
+
+    def __getinitargs__(self):
+        return (self.function,
+                self.parameters,
+                tuple(sorted(
+                    list(self.kw_parameters.items()),
+                    key=lambda item: item[0])))
+
+    def __setstate__(self, state):
+        # CallWithKwargs must override __setstate__ because during pickling the
+        # kw_parameters are converted to tuple, which needs to be converted
+        # back to dict.
+        assert len(self.init_arg_names) == len(state)
+        function, parameters, kw_parameters = state
+
+        self.function = function
+        self.parameters = parameters
+        if not isinstance(kw_parameters, dict):
+            kw_parameters = dict(kw_parameters)
+        self.kw_parameters = kw_parameters
+
+    mapper_method = intern("map_call_with_kwargs")
+
+
 class Subscript(AlgebraicLeaf):
     """An array subscript.
 
     .. attribute:: aggregate
     .. attribute:: index
+    .. attribute:: index_tuple
+
+        Return :attr:`index` wrapped in a single-element tuple, if it is not already
+        a tuple.
     """
+
+    init_arg_names = ("aggregate", "index",)
+
     def __init__(self, aggregate, index):
         self.aggregate = aggregate
         self.index = index
@@ -418,6 +847,13 @@ class Subscript(AlgebraicLeaf):
     def __getinitargs__(self):
         return self.aggregate, self.index
 
+    @property
+    def index_tuple(self):
+        if isinstance(self.index, tuple):
+            return self.index
+        else:
+            return (self.index,)
+
     mapper_method = intern("map_subscript")
 
 
@@ -426,6 +862,8 @@ class Lookup(AlgebraicLeaf):
     attribute of a class.
     """
 
+    init_arg_names = ("aggregate", "name",)
+
     def __init__(self, aggregate, name):
         self.aggregate = aggregate
         self.name = name
@@ -441,13 +879,15 @@ class Lookup(AlgebraicLeaf):
 # {{{ arithmetic primitives
 
 class _MultiChildExpression(Expression):
+    init_arg_names = ("children",)
+
     def __init__(self, children):
         assert isinstance(children, tuple)
 
         self.children = children
 
     def __getinitargs__(self):
-        return self.children
+        return self.children,
 
 
 class Sum(_MultiChildExpression):
@@ -485,7 +925,7 @@ class Sum(_MultiChildExpression):
             return self
         return Sum(self.children + (-other,))
 
-    def __nonzero__(self):
+    def __bool__(self):
         if len(self.children) == 0:
             return True
         elif len(self.children) == 1:
@@ -494,6 +934,8 @@ class Sum(_MultiChildExpression):
             # FIXME: Right semantics?
             return True
 
+    __nonzero__ = __bool__
+
     mapper_method = intern("map_sum")
 
 
@@ -526,16 +968,20 @@ class Product(_MultiChildExpression):
             return self
         return Product((other,) + self.children)
 
-    def __nonzero__(self):
+    def __bool__(self):
         for i in self.children:
             if is_zero(i):
                 return False
         return True
 
+    __nonzero__ = __bool__
+
     mapper_method = intern("map_product")
 
 
 class QuotientBase(Expression):
+    init_arg_names = ("numerator", "denominator",)
+
     def __init__(self, numerator, denominator=1):
         self.numerator = numerator
         self.denominator = denominator
@@ -551,9 +997,11 @@ class QuotientBase(Expression):
     def den(self):
         return self.denominator
 
-    def __nonzero__(self):
+    def __bool__(self):
         return bool(self.numerator)
 
+    __nonzero__ = __bool__
+
 
 class Quotient(QuotientBase):
     """
@@ -594,6 +1042,8 @@ class Power(Expression):
     .. attribute:: exponent
     """
 
+    init_arg_names = ("base", "exponent",)
+
     def __init__(self, base, exponent):
         self.base = base
         self.exponent = exponent
@@ -609,15 +1059,17 @@ class Power(Expression):
 # {{{ shift operators
 
 class _ShiftOperator(Expression):
+    init_arg_names = ("shiftee", "shift",)
+
     def __init__(self, shiftee, shift):
         self.shiftee = shiftee
         self.shift = shift
 
     def __getinitargs__(self):
-        return self.base, self.exponent
+        return self.shiftee, self.shift
 
 
-class LeftShift(Expression):
+class LeftShift(_ShiftOperator):
     """
     .. attribute:: shiftee
     .. attribute:: shift
@@ -626,7 +1078,7 @@ class LeftShift(Expression):
     mapper_method = intern("map_left_shift")
 
 
-class RightShift(Expression):
+class RightShift(_ShiftOperator):
     """
     .. attribute:: shiftee
     .. attribute:: shift
@@ -644,6 +1096,8 @@ class BitwiseNot(Expression):
     .. attribute:: child
     """
 
+    init_arg_names = ("child",)
+
     def __init__(self, child):
         self.child = child
 
@@ -687,26 +1141,27 @@ class BitwiseAnd(_MultiChildExpression):
 
 # {{{ comparisons, logic, conditionals
 
-class ComparisonOperator(Expression):
+class Comparison(Expression):
     """
     .. attribute:: left
     .. attribute:: operator
+
+        One of ``[">", ">=", "==", "!=", "<", "<="]``.
+
     .. attribute:: right
 
     .. note::
 
         Unlike other expressions, comparisons are not implicitly constructed by
-        comparing :class:`Expression` objects.
+        comparing :class:`Expression` objects. See :meth:`Expression.eq`.
     """
 
-    def __init__(self, left, operator, right):
-        """
-        :arg operator: One of ``[">", ">=", "==", "!=", "<", "<="]``.
-        """
+    init_arg_names = ("left", "operator", "right")
 
+    def __init__(self, left, operator, right):
         self.left = left
         self.right = right
-        if not operator in [">", ">=", "==", "!=", "<", "<="]:
+        if operator not in [">", ">=", "==", "!=", "<", "<="]:
             raise RuntimeError("invalid operator")
         self.operator = operator
 
@@ -721,6 +1176,8 @@ class LogicalNot(Expression):
     .. attribute:: child
     """
 
+    init_arg_names = ("child",)
+
     def __init__(self, child):
         self.child = child
 
@@ -756,8 +1213,11 @@ class If(Expression):
     .. attribute:: then
     .. attribute:: else_
     """
-    def __init__(self, criterion, then, else_):
-        self.condition = criterion
+
+    init_arg_names = ("condition", "then", "else_")
+
+    def __init__(self, condition, then, else_):
+        self.condition = condition
         self.then = then
         self.else_ = else_
 
@@ -768,6 +1228,8 @@ class If(Expression):
 
 
 class IfPositive(Expression):
+    init_arg_names = ("criterion", "then", "else_")
+
     def __init__(self, criterion, then, else_):
         from warnings import warn
         warn("IfPositive is deprecated, use If( ... >0)", DeprecationWarning,
@@ -784,11 +1246,13 @@ class IfPositive(Expression):
 
 
 class _MinMaxBase(Expression):
+    init_arg_names = ("children",)
+
     def __init__(self, children):
         self.children = children
 
     def __getinitargs__(self):
-        return self.children
+        return (self.children,)
 
 
 class Min(_MinMaxBase):
@@ -806,16 +1270,27 @@ class Max(_MinMaxBase):
 class Vector(Expression):
     """An immutable sequence that you can compute with."""
 
+    init_arg_names = ("children",)
+
     def __init__(self, children):
         assert isinstance(children, tuple)
         self.children = children
 
-    def __nonzero__(self):
+        from warnings import warn
+        warn("pymbolic vectors are deprecated in favor of either "
+                "(a) numpy object arrays and "
+                "(b) pymbolic.geometric_algebra.MultiVector "
+                "(depending on the required semantics)",
+                DeprecationWarning)
+
+    def __bool__(self):
         for i in self.children:
             if is_nonzero(i):
                 return False
         return True
 
+    __nonzero__ = __bool__
+
     def __len__(self):
         return len(self.children)
 
@@ -855,8 +1330,9 @@ class Vector(Expression):
         return Vector(tuple(other*x for x in self))
 
     def __div__(self, other):
+        # Py2 only
         import operator
-        return Vector(tuple(operator.div(x, other) for x in self))
+        return Vector(tuple(operator.div(x, other) for x in self))  # noqa pylint: disable=no-member
 
     def __truediv__(self, other):
         import operator
@@ -871,7 +1347,7 @@ class Vector(Expression):
     mapper_method = intern("map_vector")
 
 
-class cse_scope:
+class cse_scope:  # noqa
     """Determines the lifetime for the saved value of a :class:`CommonSubexpression`.
 
     .. attribute:: EVALUATION
@@ -909,6 +1385,8 @@ class CommonSubexpression(Expression):
     See :class:`pymbolic.mapper.c_code.CCodeMapper` for an example.
     """
 
+    init_arg_names = ("child", "prefix", "scope")
+
     def __init__(self, child, prefix=None, scope=None):
         """
         :arg scope: Defaults to :attr:`cse_scope.EVALUATION` if given as *None*.
@@ -939,6 +1417,8 @@ class CommonSubexpression(Expression):
 class Substitution(Expression):
     """Work-alike of sympy's Subs."""
 
+    init_arg_names = ("child", "variables", "values")
+
     def __init__(self, child, variables, values):
         self.child = child
         self.variables = variables
@@ -953,6 +1433,8 @@ class Substitution(Expression):
 class Derivative(Expression):
     """Work-alike of sympy's Derivative."""
 
+    init_arg_names = ("child", "variables")
+
     def __init__(self, child, variables):
         self.child = child
         self.variables = variables
@@ -966,6 +1448,8 @@ class Derivative(Expression):
 class Slice(Expression):
     """A slice expression as in a[1:7]."""
 
+    init_arg_names = ("children",)
+
     def __init__(self, children):
         assert isinstance(children, tuple)
         self.children = children
@@ -976,9 +1460,11 @@ class Slice(Expression):
     def __getinitargs__(self):
         return (self.children,)
 
-    def __nonzero__(self):
+    def __bool__(self):
         return True
 
+    __nonzero__ = __bool__
+
     @property
     def start(self):
         if len(self.children) > 1:
@@ -1104,6 +1590,9 @@ def quotient(numerator, denominator):
 global VALID_CONSTANT_CLASSES
 global VALID_OPERANDS
 VALID_CONSTANT_CLASSES = (int, float, complex)
+if six.PY2:
+    VALID_CONSTANT_CLASSES += (long,)  # noqa pylint:disable=undefined-variable
+
 VALID_OPERANDS = (Expression,)
 
 try:
@@ -1193,7 +1682,7 @@ def make_common_subexpression(field, prefix=None, scope=None):
     from pymbolic.geometric_algebra import MultiVector
     if isinstance(field, MultiVector):
         new_data = {}
-        for bits, coeff in field.data.iteritems():
+        for bits, coeff in six.iteritems(field.data):
             if prefix is not None:
                 blade_str = field.space.blade_bits_to_str(bits, "")
                 component_prefix = prefix+"_"+blade_str
@@ -1229,22 +1718,33 @@ def make_common_subexpression(field, prefix=None, scope=None):
             return CommonSubexpression(field, prefix, scope)
 
 
-def make_sym_vector(name, components):
+def make_sym_vector(name, components, var_factory=Variable):
     """Return an object array of *components* subscripted
-    :class:`Variable` instances.
+    :class:`Variable` (or subclass) instances.
+
+    :arg components: Either a list of indices, or an integer representing the
+        number of indices.
+    :arg var_factory: The :class:`Variable` subclass to
+        use for instantiating the scalar variables.
+
+    For example, this creates a vector with three components::
+
+        >>> make_sym_vector("vec", 3)
+        array([Subscript(Variable('vec'), 0), Subscript(Variable('vec'), 1),
+               Subscript(Variable('vec'), 2)], dtype=object)
 
-    :param components: The number of components in the vector.
     """
-    if isinstance(components, int):
-        components = range(components)
+    from numbers import Integral
+    if isinstance(components, Integral):
+        components = list(range(components))
 
     from pytools.obj_array import join_fields
-    vfld = Variable(name)
-    return join_fields(*[vfld[i] for i in components])
+    vfld = var_factory(name)
+    return join_fields(*[vfld.index(i) for i in components])
 
 
-def make_sym_array(name, shape):
-    vfld = Variable(name)
+def make_sym_array(name, shape, var_factory=Variable):
+    vfld = var_factory(name)
     if shape == ():
         return vfld
 
@@ -1252,7 +1752,7 @@ def make_sym_array(name, shape):
     result = np.zeros(shape, dtype=object)
     from pytools import indices_in_shape
     for i in indices_in_shape(shape):
-        result[i] = vfld[i]
+        result[i] = vfld.index(i)
 
     return result
 
